import datetime
import time
import pyNut.nutDate as dat
#import pytest


#=============================================================================
# UNIT TEST
#=============================================================================
def test_fDte_formatToDate():
    dte_date = '2022-02-20'
    str_format = '%Y-%m-%d'
    dte_date = dat.fDte_formatToDate(dte_date, str_format)
    assert( isinstance(dte_date, datetime.date) )
    dte_today = dat.fDte_Today()
    str_format = 'xxxx'
    dte_date = dat.fDte_formatToDate(dte_today, str_format)
    assert( isinstance(dte_date, datetime.date) )

def test_fInt_dateDifference():
    dte_today =     dat.fDte_Today()
    dte_yesterday = dat.fDte_AddDay(dte_today, -1)
    int_diff =      dat.fInt_dateDifference(dte_today, dte_yesterday)
    assert( isinstance(int_diff, int) )
    assert( int_diff == 1 )

def test_fBl_TimeIsBetween():
    dte_now =       dat.fDte_Now()
    dte_past =      dat.fDte_AddHour(dte_now, -1)
    dte_future =    dat.fDte_AddHour(dte_now, 1)
    bl_isBtw =      dat.fBl_TimeIsBetween(dte_past, dte_future, dte_now)
    assert ( isinstance(bl_isBtw, bool) )
    assert ( bl_isBtw is True )

def test_fStr_DateToString():
    dte_now =       dat.fDte_Now()
    str_dateFormat = '%Y-%m-%d'
    str_date =      dat.fStr_DateToString(dte_now, str_dateFormat = str_dateFormat)
    assert (isinstance(str_date, str))

def test_fDte_convertExcelInteger():
    int_date = 46106
    dte_date = dat.fDte_convertExcelInteger(int_date)
    print(dte_date)
    assert (isinstance(dte_date, datetime.date))
    str_date = dat.fStr_DateToString(dte_date, str_dateFormat='%Y-%m-%d')
    assert( str_date == '2026-03-25' )
    dte_date = dat.fDte_convertExcelInteger(int_date, bl_formatDate_tuple = True)
    print(dte_date)
    print(type(dte_date))
    assert (isinstance(dte_date, time.struct_time))






try:
    from . import _lib as lib
    from . import nutOther as oth
except:
    import _lib as lib
    import nutOther as oth
os          = lib.os()
time        = lib.time()
pd          = lib.pandas()
re          = lib.re()
requests    = lib.requests()
# from urllib.request import urlopen, urlretrieve
BeautifulSoup = lib.BeautifulSoup()
unicodedata = lib.unicodedata()
selenium    = lib.selenium()
# from selenium.webdriver.support.ui import WebDriverWait
# from selenium.webdriver.common.by import By
# from selenium.webdriver.common.keys import Keys
# from selenium.webdriver.support import expected_conditions as EC



#---------------------------------------------------------------
#------------- CLASS API ----------------------------
#---------------------------------------------------------------
class C_API:
    def __init__(self, d_auth={}, d_headers={}):
        # d_headers = {'User-Agent': 'Chrome/71.0.3578.98'}
        # d_headers = {"User-Agent":"Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_5) Safari/605.1.15",
        #            "Accept-Language": "en-gb",
        #            "Accept-Encoding":"br, gzip, deflate",
        #            "Accept":"test/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
        #            "Referer":"http://www.google.com/"}
        # d_headers = {'Authorization': '...'}
        self.d_credentials = {}
        self.d_headers =    d_headers
        self.d_auth =       d_auth
    def api_connect_json(self, str_url, bl_raiseErrorIfNoPage = True, bl_verify = True):
        # Existing Connection
        str_auth =      oth.fDic_GetStrFromDic(self.d_auth)
        str_headers =   oth.fDic_GetStrFromDic(self.d_headers)
        if (str_url, str_auth, str_headers) in self.d_credentials:
            self.j_resp = self.d_credentials[ (str_url, str_auth, str_headers) ]
            return self.j_resp
        # NEW Connection
        self.__url = str_url
        self.__verify = bl_verify
        # Process
        self.api_connect()
        self.api_checkConnexion(bl_raiseIfFalse = bl_raiseErrorIfNoPage)
        self.api_jsonBuild()
        # Save for later
        d_credentials = self.d_credentials
        d_credentials[ (str_url, str_auth, str_headers) ] = self.j_resp
        return self.j_resp
    def api_connect(self):
        d_param = {}
        if not self.d_auth == {}:       d_param['auth'] = self.d_auth
        if not self.d_headers == {}:    d_param['headers'] = self.d_headers
        if self.__verify is False:      d_param['verify'] = False
        try:
            o_page = requests.get(self.__url, **d_param)
        except Exception as err:
            print(' ERROR in api_connect: requests.get(str_url)')
            print(' - Type of error: ', type(err).__name__)
            print(' - ', err)
            print(' - ', self.__url)
            print(' - ', oth.fDic_GetStrFromDic(d_param), '\n')
            raise
        self.o_page = o_page
        return o_page
    def api_checkConnexion(self, bl_raiseIfFalse = False):
        try:
            if self.o_page.status_code == 200:
                return True
        except Exception as err:
            print('  ** ERROR in api_checkConnexion: {}'.format(err))
        print(' ERROR in api_checkConnexion, Connexion close for the status code of the page is not 200')
        print(' - Status code of the page is: {}'.format(self.o_page.status_code))
        print(' - URL is: ', self.__url, '\n')
        # Retour
        if bl_raiseIfFalse is True: raise
        else:                       return False
    def api_jsonBuild(self):
        try:
            j_resp = self.o_page.json()
        except:
            print(' ERROR in api_jsonBuild: json')
            print(' - ', self.__url, '\n')
            raise
        self.j_resp = j_resp
        return j_resp
    # -------------------------------------------------------------------

    def api_returnDataFrame(self, l_url_keyword=[], str_furtherSplit = ''):
        self.bl_errorOnRoll = False
        self.api_rollOnKeyword(l_url_keyword)
        self.api_rollOn_List_ofDic(str_furtherSplit)
        self.api_jsonToDataframe()
        return self.df_return
    def api_rollOnKeyword(self, l_url_keyword=[]):
        self.l_url_keyword = l_url_keyword
        j_resp = self.j_resp
        j_tableau = j_resp
        try:
            if l_url_keyword:
                for keyW in l_url_keyword:
                    if not keyW == '':
                        j_tableau = j_tableau[keyW]
        except:
            print(' INFO (API): there is no data on URL: {}'.format(self.__url))
            print('     *** with the keyword: |{}|'.format('|'.join(l_url_keyword)), '\n')
            self.bl_errorOnRoll = True
            return False
        self.j_tableau = j_tableau
    def api_rollOn_List_ofDic(self, str_furtherSplit=''):
        if str_furtherSplit == '':
            return True
        str_keyOfDestDic =  str_furtherSplit.split(':')[-1]
        str_furtherSplit =  str_furtherSplit.split(':')[0]
        str_key =           str_furtherSplit.split('=')[0]
        str_valueToChoose = str_furtherSplit.split('=')[1]
        j_list = self.j_tableau
        l_result = []
        try:
            for elem in j_list:
                if isinstance(elem, dict):
                    if elem[str_key] == str_valueToChoose:
                        d_result = elem[str_keyOfDestDic]
                        for k, v in d_result.items():
                            l_result.append({str_valueToChoose : k, str_keyOfDestDic : v})
                        self.j_tableau = l_result
                        return True
                else:   print('   ***INFO : api_rollOnList_ofDic only works with List of dictionary')
        except:
            print(' WARNING : api_rollOnList_ofDic,  str_key: |{}| and the value: |{}|'.format(str_key, str_valueToChoose))
            print('  - URL: |{}|'.format(self.__url), '\n')
            self.bl_errorOnRoll = True
            return False
        return True

    def api_jsonToDataframe(self):
        if self.bl_errorOnRoll is True:
            df_returnError = pd.DataFrame(columns=range(0, 3))
            df_returnError.loc[len(df_returnError)] = [self.__url, str('||'.join(self.l_url_keyword)), 'NULL']
            self.df_return = df_returnError
            return df_returnError
        # Transform to DATAFRAME
        try:
            try:
                df_return = pd.DataFrame(self.j_tableau)
            except ValueError:
                df_return = pd.DataFrame(self.j_tableau, index=[0])
        except Exception as err:
            print(' ERROR in api_jsonToDataframe: pd.DataFrame')
            print(' - Type of error: ', type(err).__name__)
            print(' - ', err)
            print(' - ', self.__url)
            print(' - ', self.l_url_keyword, '\n')
            raise
        self.df_return = df_return
        return df_return
#______________________________________________________________________________

#@oth.dec_singletonsClass
class C_API_simple(C_API):
    def __init__(self, d_auth={}, d_headers={}):
        super().__init__(d_auth, d_headers)

#@oth.dec_singletonsClass
class C_API_checkIndex(C_API):
    def __init__(self, d_auth={}, d_headers={}):
        super().__init__(d_auth, d_headers)
    def api_con_Check(self, str_url, bl_raiseErrorIfNoPage = True, bl_verify = True, d_check = {}):
        self.d_check = d_check
        self.api_connect_json(str_url, bl_raiseErrorIfNoPage = bl_raiseErrorIfNoPage, bl_verify = bl_verify)
        self.api_checkIndexPos()
        return self.j_resp
    def api_checkIndexPos(self):
        d_check = self.d_check
        try:
            if d_check:
                bl_check = self.fBl_Check_indexPosition(self.j_resp, d_check = d_check)
                if not bl_check:    print(' - ', self.__url)
        except Exception as err:
            print(' ERROR in api_checkIndexPos: Check')
            print(' - Type of error: ', type(err).__name__)
            print(' - ', err)
            print(' - ', self.__url)
            print(' - ', d_check, '\n')
    def fBl_Check_indexPosition(self, j_resp, l_url_keyword=['result', 'indexPositions'], d_check = {}):
        str_indexRic = d_check['str_indexRic']
        dte_url = d_check['dte_url']
        if l_url_keyword:
            for keyW in l_url_keyword:
                j_resp = j_resp[keyW]
        df_index_positions = pd.DataFrame(j_resp)
        dte_pos_isopen = df_index_positions.loc[df_index_positions['Ric'] == str_indexRic, 'IsOpen'].values[0]
        dte_pos_pubdate = \
        df_index_positions.loc[df_index_positions['Ric'] == str_indexRic, 'AsAtDate'].reset_index(drop=True)[0]
        if dte_pos_isopen != 1:
            print('\n WARNING: dte_pos_isopen is not 1 \n')
            return False
        elif dte_url != dte_pos_pubdate:
            print('\n WARNING: Index publication date incorrect for Underlying Index: ', dte_pos_pubdate, dte_url)
            print(type(dte_pos_pubdate), type(dte_url), '\n')
            return False
        else:
            return True
#______________________________________________________________________________



#=============================================================================
# Launch the CLASS
#=============================================================================
def fDf_Launch_APIclass(str_url, d_auth = {}, d_headers = {}, l_url_keyword = [], str_furtherSplit = '', bl_raiseErrorIfNoPage = True, bl_verify = True, d_check = {}):
    if d_check == {}:
        inst_getAPI = C_API_simple(d_auth, d_headers)
        inst_getAPI.api_connect_json(str_url, bl_raiseErrorIfNoPage = bl_raiseErrorIfNoPage, bl_verify = bl_verify)
    else:
        inst_getAPI = C_API_checkIndex(d_auth, d_headers)
        inst_getAPI.api_con_Check(str_url, bl_raiseErrorIfNoPage=bl_raiseErrorIfNoPage, bl_verify=bl_verify, d_check = d_check)
    inst_getAPI.api_returnDataFrame(l_url_keyword = l_url_keyword, str_furtherSplit = str_furtherSplit)
    df = inst_getAPI.df_return
    return df

@oth.dec_stopProcessTimeOut(int_secondesLimit = 30, returnIfTimeOut = False)
def fDf_Launch_APIclass_timeout(str_url, d_auth = {}, d_headers = {}, l_url_keyword = [], str_furtherSplit = '',bl_raiseErrorIfNoPage = True, bl_verify = True, d_check = {}):
    df = fDf_Launch_APIclass(str_url, d_auth = d_auth, d_headers = d_headers, l_url_keyword = l_url_keyword, str_furtherSplit = str_furtherSplit,
                             bl_raiseErrorIfNoPage = bl_raiseErrorIfNoPage, bl_verify = bl_verify, d_check = d_check)
    return df






#=============================================================================
# FUNCTION
#=============================================================================
def Act_WaitTranslation(int_sec = 5):
    print('  * Wait for Translation {} secondes ...'.format(str(int_sec)))
    time.sleep(int_sec)

def fBl_ChineseInString(str_stringToTest):
    l_result = re.findall(r'[\u4e00-\u9fff]+', str_stringToTest)
    if l_result:    return True
    return False

def fBL_checkConnexion(o_page):
    try: 
        if o_page.status_code == 200: 
            return True
        else: 
            print(' Connexion close for the status code of the page is not 200 ' )
            print(' - Status code of the page is: ' + o_page.status_code)
    except: 
        print('  ERROR in fBL_checkConnexion: Connexion fails because the input is not a page')
    return False





#==============================================================================
# BEAUTIFUL SOUP
#==============================================================================
def fDf_htmlGetArray_Soup(str_url, bl_th = False, bl_waitForTranslation = False, int_waitTime = 0, bl_cleanXA0 = True, int_waitTimeLimit = 5):  
    arr_result = []
    try:
        d_headers = {'User-Agent': 'Chrome/71.0.3578.98'}
        o_page = requests.get(str_url, headers = d_headers)
        if bl_waitForTranslation and int_waitTime != 0:     Act_WaitTranslation(int_waitTime)
    except:
        print(' ERROR in fDf_htmlGetArray_Soup: requests.get(str_url)')
        print(' -  ', str_url)
        raise
    if not fBL_checkConnexion(o_page): 
        print(' ERROR in fDf_htmlGetArray_Soup: fBL_checkConnexion(o_page): ')
        print(' -  ', str_url)
        return False    
    try:
        bs_soup = BeautifulSoup(o_page.content, "html.parser")      # lxml   # html5lib
        #bs_soup = bs_soup.replace(u'\xa0', ' ')
        #bs_soup.prettify(formatter = lambda x: x.replace(u'\xa0', ' '))
        #bs_soup.prettify(formatter = lambda x: x.replace(r'&nbsp;', ' '))
    except:
        print(' ERROR in fDf_htmlGetArray_Soup: BeautifulSoup(o_page.content, "html.parser")')
        print(' -  ', str_url)
        raise
    try:
        l_doublon = []
        for o_table in  bs_soup.find_all('table'):
            for o_row in o_table.find_all('tr'):
                # Remove doublons
                if o_row in l_doublon:
                    break
                l_doublon.append(o_row)
                # Balise Th = Text / Titre
                o_th = [o_cell.text.strip() for o_cell in o_row.find_all('th')]                
                if bl_th and o_th:      o_cells = o_th
                else:                   o_cells = []
                # Balise TD = Chiffre
                o_td = [o_cell.text.strip() for o_cell in o_row.find_all('td')]
                if o_td:                o_cells = o_cells + o_td
                elif o_th:              o_cells = o_th
                else:                   o_cells = []    
                # Clean Cells
                if bl_cleanXA0:
                    o_cells = [unicodedata.normalize("NFKD",cel_Text) for cel_Text in o_cells]
                    o_cells = [cel_Text.replace('\n', '  ').replace('\r', '') for cel_Text in o_cells]
                # add the row to result
                if o_cells:   arr_result.append(o_cells)
                # Chinese Translation - Recursive                
                if bl_waitForTranslation:
                    for cell in o_cells:
                        if fBl_ChineseInString(cell):
                            if int_waitTime > int_waitTimeLimit:
                                print('   *_* ERROR : still Chinese within Result: ', cell)
                                print('   *_!!!_* Cannot wait anymore, Do it manually: ')
                                print('   *_URL_* ', str_url)
                                break
                            else:
                                int_waitTime = (int_waitTime + 1)*2
                            print('   *_* ERROR : still Chinese in Result: ', cell)
                            df_return = fDf_htmlGetArray_Soup(str_url, bl_th, True, int_waitTime)
                            return df_return
        df = pd.DataFrame(arr_result)
        l_doublon = []
    except Exception as err:
        print(' ERROR in fDf_htmlGetArray_Soup: LOOP on tables / rows / cells')
        print(' - ', str_url)
        print(' -  Erreur: ', err)
        raise
    return df



#==============================================================================
# SELENIUM
#==============================================================================
class c_Selenium_InteractInternet():
    # ----------------------------------------------------
    # To use Chrome Driver
    #  Go to chromedriver.chromium.org
    #  download and UnZip the folder
    #  Move it to Users/local/bin or C:\ProgramData\Anaconda3\Library\bin (Windows)
    # ----------------------------------------------------
    def __init__(self, str_url):
        o_options = self.getOptions_stopMessage()
        self.driver = selenium.webdriver.Chrome(options = o_options)
        self.driver.get(str_url)
        self.realButtonName = ''
        
    def getOptions_stopMessage(self):
        options = selenium.webdriver.ChromeOptions()
        options.add_experimental_option('excludeSwitches', ['enable-logging'])
        return options
        
    def findElementByXpath(self, str_buttonxPath):
        time.sleep(1)
        try:    btn_click = self.driver.find_element_by_xpath(str_buttonxPath)
        except Exception as err:
            print(' INFO : Could not do the clic {}   ... Will wait 5 secondes'.format(err))
            time.sleep(5)
            btn_click = self.driver.find_element_by_xpath(str_buttonxPath)
        return btn_click
   
    def clic(self, str_buttonName, str_buttonxPath, l_buttonIfFailed):
        # ----------------------------------------------------
        # Right click on the button and chose Inspect
        # Spot the button Type
        # Right Click and Copy XPath, You get the XPATH
        # ----------------------------------------------------
        btn_click = self.findElementByXpath(str_buttonxPath)
        realButtonName = str(btn_click.text)
        self.realButtonName = realButtonName
        if not str_buttonName.lower() in realButtonName.lower():
            print('Link found was not |{}| but: |{}|'.format(str_buttonName,realButtonName))
            for xPath in l_buttonIfFailed:
                time.sleep(5)
                self.clic(str_buttonName, xPath, [x for x in l_buttonIfFailed if x != xPath])
                # btn_click = self.driver.find_element_by_xpath(xPath)
                # print('Link is: {}'.format(btn_click.text))
                # btn_click.click()
        else:   btn_click.click()
        
    def fillUp(self, str_buttonxPath, str_textToFill):
        time.sleep(2)
        fld_toFill = self.driver.find_element_by_xpath(str_buttonxPath)
        fld_toFill.send_keys(str_textToFill)
    
    def changeWindow(self, int_nbWindow):
        self.baseWindow = self.driver.window_handles[0]
        int_nbWindow = int_nbWindow % len(self.driver.window_handles)
        self.newWindow = self.driver.window_handles[int_nbWindow]
        self.driver.switch_to.window(self.newWindow)
        
    def changeWindowBack(self):
        try:        self.driver.switch_to.window(self.baseWindow)
        except:
            print('Could not go back to Base Window... Wron use of changeWindowBack... Make sure u used changeWindow before')
            self.driver.switch_to.window(self.driver.window_handles[0])

    def sel_quit(self):
        time.sleep(1)
        try:                        self.driver.close()
        except Exception as err:    print(' WARNING: selenium could not close: {}'.format(err))
        try:                        self.driver.quit()
        except Exception as err:    print(' WARNING: selenium could not quit: {}'.format(err))


def selenium_clicLaunch(str_url, str_buttonxPath, str_buttonName):
    try:
        inst_sel = c_Selenium_InteractInternet(str_url)
        inst_sel.clic(str_buttonName, str_buttonxPath, [])
        inst_sel.sel_quit()
        return True
    except Exception as err:
        print(' ERROR in selenium_clicLaunch: {}'.format(err))
        print(' -  str_url: {}'.format(str_url))
        print(' -  str_buttonxPath: {}'.format(str_buttonxPath))
        print(' -  str_buttonName: {}'.format(str_buttonName))
        try:    inst_sel.sel_quit()
        except: pass
        return False






try:
    from . import _lib as lib
    from . import nutFiles as fl
    from . import nutDate as dat
    from . import nutOther as oth
except:
    import _lib as lib
    import nutFiles as fl
    import nutDate as dat
    import nutOther as oth
os  = lib.os()
pd  = lib.pandas()
db  = lib.pyodbc()
lite= lib.sqlite3()






#---------------------------------------------------------------
#------------- CLASS DB management -----------------------------
#---------------------------------------------------------------




import pathlib
from setuptools import setup, find_packages
import pyNut

# The text of the README file
ME_PATH = pathlib.Path(__file__).parent
README = (ME_PATH / "README.md").read_text()

# This call to setup() does all the work
setup(
    name    = "pynut",
    version = pyNut.__version__,
    description = "Function easing life :)",
    long_description = README,
    long_description_content_type = "text/markdown",
    url = "https://github.com/Laurent-Tupin/pynut",
    author = "Laurent Tupin",
    author_email = "laurent.tupinn@gmail.com",
    license = "Copyright 2022-2035",
    classifiers=[
        "License :: Free For Home Use",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.9",
    ],
    packages = find_packages(exclude = ("test",)),
    include_package_data = True,
    install_requires = ["pandas==1.1.3", "paramiko==2.6.0", "beautifulsoup4==4.7.1", "exchangelib==4.7.2",
                        "openpyxl==3.0.5","psutil==5.9.0", "pyodbc==4.0.32", "pywin32", "selenium==3.141.0",
                        "xlrd==1.2.0", "XlsxWriter==1.3.5", "xlwings==0.20.8", "pyarrow", "fastparquet"]
    #entry_points={"console_scripts": ["EXEnameFile=FolderName.__main__:main"]},
)



import os
import pandas as pd
import numpy as np
# import pytest
try:
    import nutDataframe as dframe
except:
    print('Online Test...')
    import pyNut.nutDataframe as dframe


#=============================================================================
# UNIT TEST
#=============================================================================
def test_fDf_createSimpleDataframe():
    df_simple = dframe.fDf_createSimpleDataframe()
    assert( isinstance(df_simple, pd.DataFrame) )

def test_fBl_isDataframeEmpty():
    df_simple = dframe.fDf_createSimpleDataframe()
    bl_isempty = dframe.fBl_isDataframeEmpty(df_simple)
    assert( bl_isempty is False )

def test_fBl_compareDfCol():
    df_1 = dframe.fDf_createSimpleDataframe(l_column=['col_Join', 'data1'],
                                            l_values=[['0', 0], ['1', 1], ['2', 2], ['3', 3], ['4', 4], ['5', 5],
                                                      ['6', 6], ['7', 7], ['8', 8], ['9', 9], ['10', 10]])
    df_2 = dframe.fDf_createSimpleDataframe(l_column=['col_Join', 'data2'],
                                            l_values=[['0', 0], ['1', 1], ['2', 2], ['3', 3], ['4', 4], ['5', 5],
                                                      ['6', 6], ['7', 7], ['8', 8], ['9', 9], ['10', 10]])
    bl_compare, df_compare = dframe.fBl_compareDfCol({'df': df_1, 'colJoin': 'col_Join','colToCompare':'data1'},
                                                     {'df': df_2, 'colJoin': 'col_Join','colToCompare':'data2'})
    assert( bl_compare is True )
    assert( df_compare is None )
    df_2 = dframe.fDf_createSimpleDataframe(l_column=['col_Join', 'data1'],
                                            l_values=[['0', 0], ['1', 1], ['2', 2], ['3', 3], ['4', 4], ['5', 5],
                                                      ['6', 6], ['7', 7], ['8', 8], ['9', 9], ['10', 10]])
    bl_compare, df_compare = dframe.fBl_compareDfCol({'df': df_1, 'colJoin': 'col_Join', 'colToCompare': 'data1'},
                                                     {'df': df_2, 'colJoin': 'col_Join', 'colToCompare': 'data1'})
    assert (bl_compare is True)
    assert (df_compare is None)


def test_round_down():
    df_1 = dframe.fDf_createSimpleDataframe(l_column=['col_Join', 'RightData', 'DataToBeRounded'],
                                            l_values=[[0, 0, 0.1], [1, 0, 0.9], [2, 1, 1.5], [3, 1, 1.999], [4, 3, 3.9], [5, 2, 2],
                                                      [6, -4, -3.9], [7, -1, -0.1], [8, -2, -1.0001], [9, -7, -7], [10, -8, -7.00001]])
    df_2 = df_1.copy()
    df_2['DataRounded'] = df_2['DataToBeRounded'].apply(lambda x: dframe.round_down(x))
    bl_compare, df_compare = dframe.fBl_compareDfCol({'df': df_1, 'colJoin': 'col_Join', 'colToCompare': 'RightData'},
                                                     {'df': df_2, 'colJoin': 'col_Join', 'colToCompare': 'DataRounded'})
    assert (bl_compare is True)
    assert (df_compare is None)


def test_round_up():
    df_1 = dframe.fDf_createSimpleDataframe(l_column=['col_Join', 'RightData', 'DataToBeRounded'],
                                            l_values=[[0, 1, 0.1], [1, 1, 0.9], [2, 2, 1.5], [3, 2, 1.999], [4, 4, 3.5], [5, 2, 2],
                                                      [6, -3, -3.9], [7, 0, -0.1], [8, -1, -1.9999], [9, -7, -7], [10, -7, -7.00001]])
    df_2 = df_1.copy()
    df_2['DataRounded'] = df_2['DataToBeRounded'].apply(lambda x: dframe.round_up(x))
    bl_compare, df_compare = dframe.fBl_compareDfCol({'df': df_1, 'colJoin': 'col_Join', 'colToCompare': 'RightData'},
                                                     {'df': df_2, 'colJoin': 'col_Join', 'colToCompare': 'DataRounded'})
    assert (bl_compare is True)
    assert (df_compare is None)

def test_fBl_IsNan():
    nanValue = np.nan
    bl_nan = dframe.fBl_IsNan(nanValue)
    assert (bl_nan is True)
    df = dframe.fDf_createSimpleDataframe(l_column=['col1', 'col2', 'col3'],
                                          l_values=[[0, 1, 0.1], [2, np.nan, np.nan], [3, 2, 1.999], [4, 4, np.nan],
                                                    [np.nan, -3, -3.9], [8, np.nan, -1.9999], [9, -7, np.nan]])
    df_1 = df[df['col1'].apply(lambda x: dframe.fBl_IsNan(x))]
    df_2 = df[df['col2'].apply(lambda x: dframe.fBl_IsNan(x))]
    df_3 = df[df['col3'].apply(lambda x: dframe.fBl_IsNan(x))]
    assert (len(df_1) == 1)
    assert (len(df_2) == 2)
    assert (len(df_3) == 3)

def test_fDf_readCsv_enhanced():
    str_path = os.path.dirname(os.path.abspath(__file__))
    str_path1 = str_path + r'\fileToTest\NICB Detailed Portfolio Valuation Report 06Apr22.csv'
    df_data = dframe.fDf_readCsv_enhanced(str_path1, bl_header = None, l_names=range(13))
    assert (isinstance(df_data, pd.DataFrame) )
    str_path2 = str_path + r'\fileToTest\20211026_20211026_10740_718708NETRCNH'
    df_data = dframe.fDf_readCsv_enhanced(str_path2, bl_header=None, str_sep='|', l_names=range(13))
    assert (isinstance(df_data, pd.DataFrame))

def test_fDf_removeDoublons():
    df1 = dframe.fDf_createSimpleDataframe(l_column=['col1', 'col2', 'col3'],
                                           l_values=[[0, 1, 0.1], [2, np.nan, np.nan], [0, 1, 0.1], [0, 1, 4],
                                                     [np.nan, -3, -3.9], [0, 1, 0.1], [9, -7, np.nan]])
    df2 = dframe.fDf_removeDoublons(df1)
    assert (len(df2) == len(df1) - 2)
    assert (2 not in list(df2.index))
    assert (1 in list(df2.index))

def test_fDf_DropRowsIfNa_resetIndex():
    df1 = dframe.fDf_createSimpleDataframe(l_column=['col1', 'col2', 'col3'],
                                           l_values=[[0, 1, 0.1], [2, np.nan, np.nan], [0, 1, 0.1], [0, 1, 4],
                                                     [np.nan, -3, -3.9], [0, 1, 0.1], [9, -7, np.nan]])
    df2 = dframe.fDf_DropRowsIfNa_resetIndex(df1, l_colToDropNA = ['col1'])
    assert (len(df2) == len(df1) - 1)

def test_dDf_fillNaColumn():
    df1 = dframe.fDf_createSimpleDataframe(l_column=['col1', 'col2', 'col3'],
                                           l_values=[[0, 1, 0.1], [2, np.nan, np.nan], [3, 1, 0.1], [4, 1, 4],
                                                     [5, np.nan, -3.9], [6, np.nan, 0.1], [7, -7, np.nan]])
    df2 = dframe.dDf_fillNaColumn(df1, 'col2', 'col1')
    df3 = dframe.fDf_DropRowsIfNa_resetIndex(df2, l_colToDropNA=['col2'])
    assert (len(df2) == len(df3))

